/*
 *
 * Copyright (C) 2003 Chatchawan Wongsiriprasert
 *
 * Based on toshset(http://www.schwieters.org/toshset/) by Charles D. Schwieters
 * Based on tools(http://www.buzzard.org.uk/toshiba/) by Jonathan Buzzard
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
static const char PROGRAM_ID[]="$Id: tosh_bios.cpp,v 1.4 2003/07/29 01:43:38 cws Exp $";

#include "tosh_utils.h"
#include <ctype.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <time.h>

#include "toshibaIDs.hh"
#include "tosh_vars.h"
#include <map>

unsigned str2int(unsigned char* str,int digit)
{
    unsigned ret = 0;
    for(int i = 0; i < digit;++i,++str)
    {
        if ( !isdigit(*str) ) return ret;
        ret *= 10;
        ret += *str - '0';
    }
    
    return ret;
}

inline unsigned short readWord(const unsigned char* buf,unsigned addr)
{
    return * (unsigned short *)(&buf[addr]);
}

CToshVar* initToshBiosInfo()
{
    CFileHandle fid(open("/dev/mem",O_RDONLY));
    if ( fid < 0 ) 
    {
        throw std::runtime_error("can not open /dev/mem");
    }
    
    // Get BIOS/ROM data
    unsigned char* mem = (unsigned char *)
                mmap(0, 0x100000, PROT_READ, MAP_SHARED, fid,0); 
    if ( mem == MAP_FAILED ) 
    {
        throw "can not access ROM ";
    }
    
    
    const unsigned char* sig=(const unsigned char *)"TOSHIBA";
    const unsigned char* sig_match = sig;
    for(unsigned r = 0xFE000U;   r < 0x100000U; ++r)
    {
        if ( mem[r] == *sig_match ) ++sig_match;
        else sig_match = sig;
        
        if ( *sig_match == 0 ) break;
    }
    
    if ( *sig_match != 0 )
    {
        throw "No TOSHIBA signature found";
    }
    
    CToshVarGroup* ret = new CToshVarGroup;
    
     // Get id
    unsigned id = mem[0xFFFFE] << 8 | mem[0xFFFFA];
    char buf[32];
    unsigned addr;
    
    /* do we have a SCTTable machine identication number on our hands */ 
    if (id==0xfc2f) 
    {
        unsigned bx = 0xe6f5;
        
        addr = 0x0F0000 + 0xE6F5;
        unsigned cx = readWord(mem,addr);
        addr = 0x0F0009 + bx + cx;
        cx = readWord(mem,addr);
        addr = 0x0F000A + cx;
        cx = readWord(mem,addr);
        
        id = ((cx & 0xFF) << 8) + ((cx & 0xFF00) >> 8);
    }
    
    snprintf(buf,32,"%d",id);
    std::string s = buf;
    
    CToshStaticVar* var = new CToshStaticVar(buf,toshibaModelName(id));
    ret->addItem("id",var);
        
    addr = 0xFE008;
    if ( mem[addr] == 'v' ) ++addr;
    
    s.erase();
    for(;addr < 0x100000; ++addr)
    {
        if ( mem[addr] == 0 ) break;
        
        s += (char)mem[addr];
    }
    var = new CToshStaticVar(s,s);
    ret->addItem("version",var);
    
    struct tm bios_time;
    memset(&bios_time,0,sizeof(bios_time));
    bios_time.tm_year = 100 + str2int(&mem[0xFFFFB],2);
    bios_time.tm_mon =  str2int(&mem[0xFFFF5],2) - 1;
    bios_time.tm_mday = str2int(&mem[0xFFFF8],2);
    time_t t = mktime(&bios_time);

    snprintf(buf,32,"%ld",(long)t);
    s = "20";s += mem[0xFFFFB];s += mem[0xFFFFC];s += '-';
    s += mem[0xFFFF5]; s+= mem[0xFFFF6];s+= '-';
    s += mem[0xFFFF8]; s+= mem[0xFFFF9];
    
    var = new CToshStaticVar(buf,s);
    ret->addItem("date",var);
    
    return ret;
}
