/*
 * Copyright (C) 2003 Chatchawan Wongsiriprasert
 *
 * Based on toshset(http://www.schwieters.org/toshset/) by Charles D. Schwieters
 * Based on tools(http://www.buzzard.org.uk/toshiba/) by Jonathan Buzzard
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
static const char PROGRAM_ID[]="$Id: tosh_hci.cpp,v 1.5 2003/07/29 01:43:38 cws Exp $";

#include "tosh_hci.h"
#include <stdio.h>

CToshVar* initToshHCI()
{
    CToshVarGroup* ret = new CToshVarGroup();
    
    CToshHCIVar * var = new CReadOnlyToshHCIVar(CToshHCIVar::HCI_AC_ADAPTOR);
    
    var->addValue(3,0,"BATTERY");
    var->addValue(4,1,"EXTERNAL");
    ret->addItem("power_source",var);
     
    CToshVarGroup *var_group = new CToshVarGroup();
    
    var = new CToshHCIVar(CToshHCIVar::HCI_BACKLIGHT);
    var->addValue(0,0,"OFF");
    var->addValue(1,1,"ON");
    var_group->addItem("backlight",var);
    
    var_group->addItem("brightness",
            new CToshHCIVar(CToshHCIVar::HCI_LCD_BRIGNTNESS,
                                    16-3,0x00000007));
                                    
    var = new CReadOnlyToshHCIVar(CToshHCIVar::HCI_FLAT_PANEL,
                                                 8,0x000000FF);
    var->addValue(0,0,"640x480");
    var->addValue(1,1,"800x600");
    var->addValue(2,2,"1024x768");
    var->addValue(3,3,"1024x600");
    var->addValue(4,4,"800x480");
    var->addValue(5,5,"(unknown)");
    var->addValue(6,6,"1400x1050");
    var->addValue(7,7,"1600x1200");
    var_group->addItem("mode",var);
    
    var = new CReadOnlyToshHCIVar(CToshHCIVar::HCI_FLAT_PANEL,
                                                 0,0x000000FF);
    var->addValue(0,0,"STN_MONO");
    var->addValue(1,1,"STN_COLOR");
    var->addValue(2,2,"9BIT_TFT");
    var->addValue(3,3,"12BIT_TFT");
    var->addValue(4,4,"18BIT_TFT");
    var->addValue(5,5,"14BIT_TFT");
    var_group->addItem("type",var);
    
    ret->addItem("lcd",var_group);
    
    var = new CToshHCIVar(CToshHCIVar::HCI_FAN);
    var->addValue(0,0,"OFF");
    var->addValue(0x00000080,1,"LOW");
    var->addValue(0x000000FF,2,"HIGH");
    ret->addItem("fan",var);
    
    // Better suspend with apm -z
    var = new CReadOnlyToshHCIVar(CToshHCIVar::HCI_SOFTWARE_SUSPEND);
    var->addValue(0,0,"DISABLED");
    var->addValue(1,1,"ENABLED");
    ret->addItem("software_suspend",var);
    
    // CPU Speed
    var = new CToshHCIVar(CToshHCIVar::HCI_CPU_SPEED);
    var->addValue(0x00006000,0,"LOW");
    var->addValue(0,1,"HIGH");
    ret->addItem("cpu_speed",var);
    
    // SELECT bay
    var = new CReadOnlyToshHCIVar(CToshHCIVar::HCI_SELECT_STATUS);
    var->addValue(0,0,"EMPTY");
    var->addValue(1,1,"FLOPPY DISK DRIVE");
    var->addValue(2,2,"ATAPI DEVICE");
    var->addValue(3,3,"IDE DEVICE");
    var->addValue(4,4,"2nd ATAPI DEVICE");
    ret->addItem("select_bay",var);
        
    return ret;
}

//---------------------------------------------
// CToshHCIVar
//---------------------------------------------
std::string
CToshHCIVar::getErrorMsg(unsigned code)
{
    static const std::string SUCCESS("SUCCESS");
    static const std::string FAILURE("HCI Call failure");
    static const std::string NOT_SUPPORTED("System does not support this function");
    static const std::string INPUT_ERROR("Input data error");
    static const std::string WRITE_PROTECTED("Write protected");
    static const std::string FIFO_EMPTY("System event FIFO empty");
    static const std::string NOTREADY("Device is not ready");
    
    switch(code)
    {   
        case HCI_SUCCESS: return SUCCESS;
        case HCI_FAILURE: return FAILURE;
        case HCI_NOT_SUPPORTED: return NOT_SUPPORTED;
        case HCI_INPUT_ERROR: return INPUT_ERROR;
        case HCI_WRITE_PROTECTED: return WRITE_PROTECTED;
        case HCI_FIFO_EMPTY: return FIFO_EMPTY;
        case HCI_NOTREADY: return NOTREADY;
    }
    
    char buf[32];
    snprintf(buf,32,"Unknown HCI error(0x%X)",code);
    
    return buf;
}

unsigned 
CToshHCIVar::hci_read(HCI_FUNCTION fn,unsigned ecx,unsigned edx) 
                              throw (std::runtime_error)
{
    unsigned result;
    
    if ( !hci_read(result,fn,ecx,edx) ) 
                throw std::runtime_error("HCI read error -- " + getErrorMsg(result));
                
    return result;
}
                                                
void 
CToshHCIVar::hci_write(HCI_FUNCTION fn,unsigned ecx,unsigned edx) 
                                throw (std::runtime_error)
{
    unsigned result;
    
    if ( !hci_write(result,fn,ecx,edx) ) 
                throw std::runtime_error("HCI write error -- " + getErrorMsg(result));
}

bool 
CToshHCIVar::hci_call(unsigned& result,unsigned eax,unsigned ebx,
                              unsigned ecx,unsigned edx) throw()
{
    unsigned result_a,result_c;
  
    asm ("inb $0xb2,%%al\n\t"
            "jc hci_call_error\n\t"
            "movl %%ecx,%%eax\n\t"
            "xor  %%eax,%%eax\n"
            "hci_call_error:"
        : "=a" (result_a) ,"=c" (result_c)
        : "a" (eax), "b" (ebx), "c" (ecx), "d" (edx));
    
    if ( result_a != 0 )
    {
        result = (result_a & 0x0000FF00) >> 8;
        return false;
    }
    else
    {
        result = result_c;
        return true;
    }    
}

//------------------------------------------------------------
// CToshHCIVar continue
//------------------------------------------------------------
std::string 
CToshHCIVar::read(const std::string& item_name,
                            unsigned fmt) const
{
    static const std::string UNKNOWN("(unknown)");
    
    unsigned v = hci_read(0,0);
    
    if ( fmt != FMT_RAW )
    {        
        v >>= m_Shift;
        v &= m_Mask;
        
        if ( m_Values.size() > 0 )
        {
            std::map<unsigned,VAR_INFO>::const_iterator it = m_Values.find(v);
        
            if ( it == m_Values.end() )
            {
                if ( fmt == FMT_HUMAN ) return UNKNOWN;
                else v = (unsigned)-1;
            }
            else
            {
                if ( fmt == FMT_HUMAN ) return it->second.second;
                else v = it->second.first; 
            }
        }
    }
    
    char buf[32];
    snprintf(buf,sizeof(buf),"%d",v);
    return buf;
}

void 
CToshHCIVar::write(const std::string& item_name,
                              const std::string& value,
                              unsigned fmt)
{
    unsigned v = (unsigned)atoi(value.c_str());
    
    if ( fmt != FMT_RAW && fmt != FMT_STD )
    {
        throw unsupport_exception("unsupport value format"); 
    }
    
    if ( fmt == FMT_STD )
    {
        v &= m_Mask;
        v <<= m_Shift;
        
        if ( m_Values.size() > 0 )
        {
            std::map<unsigned,VAR_INFO>::const_iterator it;
            for(it = m_Values.begin(); it != m_Values.end() ; ++it)
            { if ( it->second.first == v ) break; }
        
            if ( it == m_Values.end() )
            {
                throw unsupport_exception("unsupport variable value"); 
            }
            v = it->first;
        }
    }
    
    hci_write(v,0);
}
                                
void 
CToshHCIVar::getItems(std::vector<std::string>& items) const
{
    items.empty();
}

void 
CToshHCIVar::addValue(unsigned value,unsigned show_value,
                                   const std::string& str_value)
{
    m_Values[value] = VAR_INFO(show_value,str_value);
}                                
