/*
 * Copyright (C) 2003 Chatchawan Wongsiriprasert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
static const char PROGRAM_ID[]="$Id: tosh_vars.cpp,v 1.2 2003/03/20 00:53:24 cws Exp $";

#define __TOSH_VARS_CPP__
#include "tosh_vars.h"

//------------------------------------------------------------
// CToshVarGroup
//------------------------------------------------------------
CToshVarGroup::CToshVarGroup() : CToshVar(),m_VarMap()
{
}

CToshVarGroup::~CToshVarGroup()
{
    VARMAP::iterator i;
    for(i = m_VarMap.begin(); i != m_VarMap.end(); ++i)
    {
        delete i->second;
    }
}

void
CToshVarGroup::addItem(const std::string& item_name,CToshVar* var)
{ 
    m_VarMap[item_name] = var;
}

std::string 
CToshVarGroup::read(const std::string& item_name,
                        unsigned fmt) const
{
    std::string::size_type dot_pos = item_name.find('.');
    std::string mainitem_name = item_name.substr(0,dot_pos);
    std::string subitem_name;
    if ( dot_pos != std::string::npos )
    {
        subitem_name = item_name.substr(dot_pos+1);
    }
                                                                               
    VARMAP::const_iterator it = m_VarMap.find(mainitem_name);
    if ( it == m_VarMap.end() ) 
    {
        throw std::runtime_error("Unknown item name [" + mainitem_name + ']');
    }
        
    return it->second->read(subitem_name,fmt);
}

void 
CToshVarGroup::write(const std::string& item_name,
                         const std::string& value,
                         unsigned fmt)
{
    std::string::size_type dot_pos = item_name.find('.');
    std::string mainitem_name = item_name.substr(0,dot_pos);
    std::string subitem_name;
    if ( dot_pos != std::string::npos )
    {
        subitem_name = item_name.substr(dot_pos+1);
    }
                                                                               
    VARMAP::const_iterator it = m_VarMap.find(mainitem_name);
    if ( it == m_VarMap.end() ) 
    {
        throw std::runtime_error("Unknown item name [" + mainitem_name + ']');
    }
        
    it->second->write(subitem_name,value,fmt);
}
                                
void 
CToshVarGroup::getItems(std::vector<std::string>& items,
                                      const std::string& group_name) const
{
    VARMAP::const_iterator it;
    items.empty();
    
    if ( group_name.empty() ) it = m_VarMap.begin();
    else it = m_VarMap.find(group_name);
    
    for(; it != m_VarMap.end(); ++it)
    {
        std::vector<std::string> v;
        it->second->getItems(v,std::string());
        
        if ( v.size() == 0 )
        {    
            items.push_back(it->first);
        }
            
        std::vector<std::string>::const_iterator j;
        for(j = v.begin(); j != v.end(); ++j )
        {
            std::string s = it->first;
                
            s+= '.';
            items.push_back(s+*j);
        }
        
        if ( !group_name.empty() ) return;
    }
}
