/*
 * $Id: tosh_vars.h,v 1.2 2003/03/20 00:53:24 cws Exp $
 * Copyright (C) 2003 Chatchawan Wongsiriprasert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef __TOSH_VARS_H__
#define __TOSH_VARS_H__

#include <stdexcept>
#include <vector>
#include <string>
#include <map>

class CToshVar
{
private:
    // GCC 2.95.3 need at least one data member with class that has 
    // virtual meber function.
    int   CToshVar_dummy_var;
    
public:
    class unsupport_exception : public std::runtime_error
    {
    public:
        unsupport_exception(const std::string& msg) : std::runtime_error(msg)
        {}
    };
    
    CToshVar() {}
    virtual ~CToshVar() {}
    
    enum READ_FORMAT { FMT_RAW = 0, FMT_STD = 1 , FMT_HUMAN = 2 };
    virtual std::string read(const std::string& item_name,unsigned fmt) const
    { throwUnsupportRead(); return std::string(); }                                               
        
    virtual void write(const std::string& item_name,
                                 const std::string& value,unsigned fmt)
    { throwUnsupportWrite(); }    
        
    virtual void getItems(std::vector<std::string>& names,
                                      const std::string& group_name) const
    { names.empty(); }                                      
    
    static void throwUnsupportRead()
    {
        throw unsupport_exception("unsupport function (read)"); 
    }
    static void throwUnsupportWrite()
    {
        throw unsupport_exception("unsupport function (write)"); 
    }
};

class CToshVarGroup : public CToshVar
{
private:
    typedef std::map<std::string,CToshVar* > VARMAP;
    VARMAP m_VarMap;
    
public:
    CToshVarGroup();
    virtual ~CToshVarGroup();
    
    void addItem(const std::string& item_name,CToshVar* var);
    virtual std::string read(const std::string& item_name,
                                          unsigned fmt) const;
    virtual void write(const std::string& item_name,
                                const std::string& value,unsigned fmt);
                                
    virtual void getItems(std::vector<std::string>& item_names,
                                     const std::string& group_name) const;
};

class CToshStaticVar : public CToshVar
{
    std::string m_StdValue;
    std::string m_HumanValue;
public:
    CToshStaticVar(const std::string& std_value,const std::string& human_value)
    : CToshVar(),m_StdValue(std_value),m_HumanValue(human_value) 
    {}
    virtual ~CToshStaticVar() {}
    
    virtual std::string read(const std::string& item_name,unsigned fmt) const
    { 
        if ( fmt == FMT_HUMAN ) return m_HumanValue;
        return m_StdValue;
    }
};

#endif
